#
# Copyright 2011 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#

import optparse

import os
import sys
import signal
from errno import *
import time
import traceback


# our modules
from albatross import simpleserver, init_logging
from albatross.cmdline.common import *


class Worker:

    def __init__(self, options):
        self.options = options
        self.server = None
        self.logger = None
        if options.logfile:
            if options.debug:
                level = 'DEBUG'
            else:
                level = 'INFO'
            self.logger = init_logging(filename=options.logfile, level=level)

    def init(self):
        self.server = simpleserver.Server(self.options.port,
                                          logger=self.logger,
                                          debug=self.options.debug)
        return self

    def run(self):
        self.server.select_loop()

    def stop(self):
        self.server.stop()


usage = '''%prog session-server [options]... <command>
        Start or stop an albatross session server'''

full_usage = usage + '''

<command> is one of:
    start    start a new daemon
    stop     kill the current daemon
    status   request the daemon's status'''


def main(args):
    optp = optparse.OptionParser(usage=full_usage)
    optp.add_option('-p', '--port', type='int', default=34343,
                    help='Listen on PORT (default: %default)')
    optp.add_option('-l', '--logfile', '--log',
                    help='Write log to LOGFILE (default: %default)')
    optp.add_option('-D', '--debug', default=False, action='store_true',
                    help='Generate additional debugging logs')
    if os.name == 'posix':
        optp.add_option('-k', '--pidfile', '--pid-file',
                        default='/var/run/al-session-server.pid',
                        help='Record server pid in PIDFILE (default: %default)')
        optp.add_option('--daemon', '--bg', action='store_true',
                        help='Run in the background')
        optp.add_option('--foreground', '--fg', 
                        dest='daemon', action='store_false',
                        help='Run in the foreground')
        optp.set_defaults(daemon=True)
    else:
        optp.set_defaults(daemon=False)
    options, args = optp.parse_args(args)

    if options.daemon:
        if not args or args[0] not in ('start', 'stop', 'status'):
            optp.error('must specify a subcommand (start, stop or status)')
        if not options.pidfile:
            optp.error('--pidfile must be specified with --daemon')
        daemon = Daemon(Worker(options), pidfile=options.pidfile)
        sys.exit(daemon.command(args[0]))
    else:
        try:
            Worker(options).init().run()
        except KeyboardInterrupt:
            sys.exit(EXIT_OK)


if __name__ == '__main__':
    main(sys.argv[1:])
