import sys
import os
import getopt

sys.path.insert(0, '..')

_ARGS_ARGS_FLAG = 4
_KW_ARGS_FLAG = 8


def find_methods(klass):
    methods = []
    for name, attr in klass.__dict__.iteritems():
        if name.startswith('_'):
            continue
        if callable(attr):
            args = []
            code = attr.func_code
            arg_names = list(code.co_varnames[1:code.co_argcount])
            arg_idx = code.co_argcount
            if code.co_flags & _ARGS_ARGS_FLAG:
                arg_names.append('*' + code.co_varnames[arg_idx])
                arg_idx += 1
            if code.co_flags & _KW_ARGS_FLAG:
                arg_names.append('**' + code.co_varnames[arg_idx])
            methods.append((name, arg_names, klass.__name__))
    for base in klass.__bases__:
        methods.extend(find_methods(base))
    return methods


def combine_methods(methods):
    combined = []
    names = {}
    for name, arg_names, klass in methods:
        if names.get(name):
            continue
        combined.append((name, arg_names, klass))
        names[name] = 1
    combined.sort()
    return combined


class TableReST:
    def __init__(self, *headings):
        self.headings = headings
        self.rows = []

    def row(self, *row):
        self.rows.append(row)

    def calc_widths(self):
        columns = zip(*([self.headings] + self.rows))
        self.colwidths = [max(map(len, col)) for col in columns]

    def write_rule(self, char='-'):
        out = ['+']
        for width in self.colwidths:
            out.append(char * (width + 2))
            out.append('+')
        print ''.join(out)

    def write_row(self, row):
        out = ['|']
        for col, width in zip(row, self.colwidths):
            out.append(' %-*s |' % (width, col))
        print ''.join(out)

    def write(self):
        self.calc_widths()
        self.write_rule('-')
        self.write_row(self.headings)
        self.write_rule('=')
        for row in self.rows:
            self.write_row(row)
            self.write_rule('-')


def method_table(klass):
    n = find_methods(klass)
    table = TableReST('Method', 'Mixin')
    for name, arg_names, klass in combine_methods(n):
        args = ', '.join(arg_names)
        table.row(':meth:`%s(%s)`' % (name, args), ':class:`%s`' % klass)
    table.write()


def usage():
    sys.exit('usage: %s [-o filename] module.class'
             % os.path.basename(sys.argv[0]))


if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'o:', ['output='])
    except getopt.GetoptError:
        usage()
    for opt, arg in opts:
        if opt in ('-o', '--output'):
            sys.stdout = open(arg, 'w')

    if len(args) != 1:
        usage()
    try:
        mod_name, klass_name = args[0].split('.')
    except:
        usage()

    mod = __import__(mod_name)
    klass = getattr(mod, klass_name)
    method_table(klass)
