from albatross.ext.form import *
from albatross.tags import escape
from util.pager import PageSelectionDisplay, DatabasePagedList
from util.db import ORDER_ASC
from contact import Contact


class ContactLinkField(Field):

    def __init__(self, name, attr, contact, html_attrs=None):
        self.contact = contact
        Field.__init__(self, name, attr, html_attrs=html_attrs)

    def write_static_html(self, ctx, form):
        form.write_content(ctx, self.get_display_value(ctx, form))

    def write_form_html(self, ctx, form):
        if self.value is None or not str(self.value).strip():
            return self.write_static_html(ctx, form)
        href = ctx.redirect_url('edit_contact?contact_id=%s' %
                                self.contact.id)
        form.write_content(ctx, self.format_tag('a', href=href))
        form.write_content(ctx, self.get_display_value(ctx, form))
        form.write_content(ctx, self.format_tag('/a'))


class EmailField(Field):

    def get_display_value(self, ctx, form):
        if not self.value or not self.value.strip():
            return '&nbsp;'
        if form.disabled:
            return escape(self.value)
        return '<a href="mailto:%s">%s</a>' % (self.value,
                                               escape(self.value))


class ContactTableRow(IteratorTableRow):

    def __init__(self, contact=None):
        self.contact = contact
        cols = [Col((col,)) for col in (
                ContactLinkField('Firstname', 'firstname', contact),
                ContactLinkField('Lastname', 'lastname', contact),
                EmailField('Email', 'email', static=True),
                ContactLinkField('Mobile', 'mobile_phone', contact),
                ContactLinkField('Office', 'office_phone', contact),
                ContactLinkField('Home', 'home_phone', contact),
                Checkbox('Delete', None),
                )]
        IteratorTableRow.__init__(self, cols)
        if contact:
            self.load(contact)


class ContactsForm(FieldsetForm):

    def __init__(self):
        self.contacts_table = IteratorTable(
            'contacts_table',
            None,
            DatabasePagedList(ContactTableRow, Contact,
                              order_args=(('lastname', ORDER_ASC),)),
            page_selection_display=PageSelectionDisplay())
        buttons = Buttons((Button('Delete selected', 'delete_selected'),
                           Button('Add contact', 'add_contact')))
        FieldsetForm.__init__(self, 'Contacts',
                              (self.contacts_table, ),
                              buttons=buttons)


class ContactForm(FieldsetForm):

    def __init__(self, title):
        elements = Fieldset((
                LabelFieldRow(TextField('Firstname', 'firstname',
                                        required=True)),
                LabelFieldRow(TextField('Lastname', 'lastname',
                                        required=True)),
                LabelFieldRow(TextField('Address', 'address')),
                LabelFieldRow(TextField('City', 'city')),
                LabelFieldRow(TextField('Postcode', 'postcode')),
                LabelFieldRow(TextField('Email', 'email')),
                LabelFieldRow(TextField('Mobile', 'mobile_phone')),
                LabelFieldRow(TextField('Office', 'office_phone')),
                LabelFieldRow(TextField('Home', 'home_phone')),))
        buttons = Buttons((Button('Save', 'save_contact'),
                           Button('Cancel', 'cancel_contact')))
        FieldsetForm.__init__(self, title,
                              (elements,), buttons=buttons)

